@@ -164,7 +164,7 @@ START_TEST(device_disable)
 	ck_assert_int_eq(status, LIBINPUT_CONFIG_STATUS_SUCCESS);
 	litest_assert_empty_queue(li);
 
-	/* event from renabled device */
+	/* event from re-enabled device */
 	litest_event(dev, EV_REL, REL_X, 10);
 	litest_event(dev, EV_SYN, SYN_REPORT, 0);
 
@@ -1419,6 +1419,95 @@ START_TEST(device_quirks_logitech_marble_mouse)
 }
 END_TEST
 
+char *debug_messages[64] = { NULL };
+
+static void
+debug_log_handler(struct libinput *libinput,
+		  enum libinput_log_priority priority,
+		  const char *format,
+		  va_list args)
+{
+	char *message;
+	int n;
+
+	if (priority != LIBINPUT_LOG_PRIORITY_DEBUG)
+		return;
+
+	n = xvasprintf(&message, format, args);
+	litest_assert_int_gt(n, 0);
+
+	for (size_t idx = 0; idx < ARRAY_LENGTH(debug_messages); idx++) {
+		if (debug_messages[idx] == NULL) {
+			debug_messages[idx] = message;
+			return;
+		}
+	}
+
+	litest_abort_msg("Out of space for debug messages");
+}
+
+START_TEST(device_quirks)
+{
+	struct libinput *li;
+	struct litest_device *dev;
+	struct libinput_device *device;
+	char **message;
+	bool disable_key_f1 = false,
+	     enable_btn_left = false;
+#if HAVE_LIBEVDEV_DISABLE_PROPERTY
+	bool disable_pointingstick = false,
+	     enable_buttonpad = false;
+#endif
+
+	li = litest_create_context();
+	libinput_log_set_priority(li, LIBINPUT_LOG_PRIORITY_DEBUG);
+	libinput_log_set_handler(li, debug_log_handler);
+	dev = litest_add_device(li, LITEST_KEYBOARD_QUIRKED);
+	device = dev->libinput_device;
+
+	ck_assert(libinput_device_pointer_has_button(device,
+						     BTN_LEFT));
+	ck_assert(libinput_device_pointer_has_button(dev->libinput_device,
+						     BTN_RIGHT));
+	ck_assert(!libinput_device_keyboard_has_key(dev->libinput_device,
+						    KEY_F1));
+	ck_assert(!libinput_device_keyboard_has_key(dev->libinput_device,
+						    KEY_F2));
+	ck_assert(!libinput_device_keyboard_has_key(dev->libinput_device,
+						    KEY_F3));
+
+	/* Scrape the debug messages for confirmation that our quirks are
+	 * triggered, the above checks cannot work non-key codes */
+	message = debug_messages;
+	while (*message) {
+		if (strstr(*message, "disabling EV_KEY KEY_F1"))
+			disable_key_f1 = true;
+		if (strstr(*message, "enabling EV_KEY BTN_LEFT"))
+			enable_btn_left = true;
+#if HAVE_LIBEVDEV_DISABLE_PROPERTY
+		if (strstr(*message, "enabling INPUT_PROP_BUTTONPAD"))
+			enable_buttonpad = true;
+		if (strstr(*message, "disabling INPUT_PROP_POINTING_STICK"))
+			disable_pointingstick = true;
+#endif
+		free(*message);
+		message++;
+	}
+
+	ck_assert(disable_key_f1);
+	ck_assert(enable_btn_left);
+#if HAVE_LIBEVDEV_DISABLE_PROPERTY
+	ck_assert(enable_buttonpad);
+	ck_assert(disable_pointingstick);
+#endif
+
+	litest_disable_log_handler(li);
+
+	litest_delete_device(dev);
+	litest_destroy_context(li);
+}
+END_TEST
+
 START_TEST(device_capability_at_least_one)
 {
 	struct litest_device *dev = litest_current_device();
@@ -1462,10 +1551,11 @@ START_TEST(device_capability_nocaps_ignored)
 	struct libinput *li;
 	struct libinput_device *device;
 
-	/* SW_MAX isn't handled in libinput so the device is processed but
-	 * ends up without seat capabilities and is ignored. */
+	/* SW_PEN_INSERTED isn't handled in libinput so the device is
+	 * processed but ends up without seat capabilities and is ignored.
+	 */
 	uinput = litest_create_uinput_device("test device", NULL,
-					     EV_SW, SW_MAX,
+					     EV_SW, SW_PEN_INSERTED,
 					     -1);
 	li = litest_create_context();
 	device = libinput_path_add_device(li,
@@ -1612,78 +1702,79 @@ TEST_COLLECTION(device)
 	struct range abs_range = { 0, ABS_MISC };
 	struct range abs_mt_range = { ABS_MT_SLOT + 1, ABS_CNT };
 
-	litest_add("device:sendevents", device_sendevents_config, LITEST_ANY, LITEST_TOUCHPAD|LITEST_TABLET);
-	litest_add("device:sendevents", device_sendevents_config_invalid, LITEST_ANY, LITEST_TABLET);
-	litest_add("device:sendevents", device_sendevents_config_touchpad, LITEST_TOUCHPAD, LITEST_TABLET);
-	litest_add("device:sendevents", device_sendevents_config_touchpad_superset, LITEST_TOUCHPAD, LITEST_TABLET);
-	litest_add("device:sendevents", device_sendevents_config_default, LITEST_ANY, LITEST_TABLET);
-	litest_add("device:sendevents", device_disable, LITEST_RELATIVE, LITEST_TABLET);
-	litest_add("device:sendevents", device_disable_tablet, LITEST_TABLET, LITEST_ANY);
-	litest_add("device:sendevents", device_disable_touchpad, LITEST_TOUCHPAD, LITEST_TABLET);
-	litest_add("device:sendevents", device_disable_touch, LITEST_TOUCH, LITEST_ANY);
-	litest_add("device:sendevents", device_disable_touch_during_touch, LITEST_TOUCH, LITEST_ANY);
-	litest_add("device:sendevents", device_disable_touch, LITEST_SINGLE_TOUCH, LITEST_TOUCHPAD);
-	litest_add("device:sendevents", device_disable_touch_during_touch, LITEST_SINGLE_TOUCH, LITEST_TOUCHPAD);
-	litest_add("device:sendevents", device_disable_events_pending, LITEST_RELATIVE, LITEST_TOUCHPAD|LITEST_TABLET);
-	litest_add("device:sendevents", device_double_disable, LITEST_ANY, LITEST_TABLET);
-	litest_add("device:sendevents", device_double_enable, LITEST_ANY, LITEST_TABLET);
-	litest_add_no_device("device:sendevents", device_reenable_syspath_changed);
-	litest_add_no_device("device:sendevents", device_reenable_device_removed);
-	litest_add_for_device("device:sendevents", device_disable_release_buttons, LITEST_MOUSE);
-	litest_add_for_device("device:sendevents", device_disable_release_keys, LITEST_KEYBOARD);
-	litest_add("device:sendevents", device_disable_release_tap, LITEST_TOUCHPAD, LITEST_ANY);
-	litest_add("device:sendevents", device_disable_release_tap_n_drag, LITEST_TOUCHPAD, LITEST_ANY);
-	litest_add("device:sendevents", device_disable_release_softbutton, LITEST_CLICKPAD, LITEST_APPLE_CLICKPAD);
-	litest_add("device:sendevents", device_disable_topsoftbutton, LITEST_TOPBUTTONPAD, LITEST_ANY);
-	litest_add("device:id", device_ids, LITEST_ANY, LITEST_ANY);
-	litest_add_for_device("device:context", device_context, LITEST_SYNAPTICS_CLICKPAD_X220);
-	litest_add_for_device("device:context", device_user_data, LITEST_SYNAPTICS_CLICKPAD_X220);
+	litest_add(device_sendevents_config, LITEST_ANY, LITEST_TOUCHPAD|LITEST_TABLET);
+	litest_add(device_sendevents_config_invalid, LITEST_ANY, LITEST_TABLET);
+	litest_add(device_sendevents_config_touchpad, LITEST_TOUCHPAD, LITEST_TABLET);
+	litest_add(device_sendevents_config_touchpad_superset, LITEST_TOUCHPAD, LITEST_TABLET);
+	litest_add(device_sendevents_config_default, LITEST_ANY, LITEST_TABLET);
+	litest_add(device_disable, LITEST_RELATIVE, LITEST_TABLET);
+	litest_add(device_disable_tablet, LITEST_TABLET, LITEST_ANY);
+	litest_add(device_disable_touchpad, LITEST_TOUCHPAD, LITEST_TABLET);
+	litest_add(device_disable_touch, LITEST_TOUCH, LITEST_ANY);
+	litest_add(device_disable_touch_during_touch, LITEST_TOUCH, LITEST_ANY);
+	litest_add(device_disable_touch, LITEST_SINGLE_TOUCH, LITEST_TOUCHPAD);
+	litest_add(device_disable_touch_during_touch, LITEST_SINGLE_TOUCH, LITEST_TOUCHPAD);
+	litest_add(device_disable_events_pending, LITEST_RELATIVE, LITEST_TOUCHPAD|LITEST_TABLET);
+	litest_add(device_double_disable, LITEST_ANY, LITEST_TABLET);
+	litest_add(device_double_enable, LITEST_ANY, LITEST_TABLET);
+	litest_add_no_device(device_reenable_syspath_changed);
+	litest_add_no_device(device_reenable_device_removed);
+	litest_add_for_device(device_disable_release_buttons, LITEST_MOUSE);
+	litest_add_for_device(device_disable_release_keys, LITEST_KEYBOARD);
+	litest_add(device_disable_release_tap, LITEST_TOUCHPAD, LITEST_ANY);
+	litest_add(device_disable_release_tap_n_drag, LITEST_TOUCHPAD, LITEST_ANY);
+	litest_add(device_disable_release_softbutton, LITEST_CLICKPAD, LITEST_APPLE_CLICKPAD);
+	litest_add(device_disable_topsoftbutton, LITEST_TOPBUTTONPAD, LITEST_ANY);
+	litest_add(device_ids, LITEST_ANY, LITEST_ANY);
+	litest_add_for_device(device_context, LITEST_SYNAPTICS_CLICKPAD_X220);
+	litest_add_for_device(device_user_data, LITEST_SYNAPTICS_CLICKPAD_X220);
 
-	litest_add("device:udev", device_get_udev_handle, LITEST_ANY, LITEST_ANY);
+	litest_add(device_get_udev_handle, LITEST_ANY, LITEST_ANY);
 
-	litest_add("device:group", device_group_get, LITEST_ANY, LITEST_ANY);
-	litest_add_no_device("device:group", device_group_ref);
-	litest_add_no_device("device:group", device_group_leak);
+	litest_add(device_group_get, LITEST_ANY, LITEST_ANY);
+	litest_add_no_device(device_group_ref);
+	litest_add_no_device(device_group_leak);
 
-	litest_add_no_device("device:invalid devices", abs_device_no_absx);
-	litest_add_no_device("device:invalid devices", abs_device_no_absy);
-	litest_add_no_device("device:invalid devices", abs_mt_device_no_absx);
-	litest_add_no_device("device:invalid devices", abs_mt_device_no_absy);
-	litest_add_ranged_no_device("device:invalid devices", abs_device_no_range, &abs_range);
-	litest_add_ranged_no_device("device:invalid devices", abs_mt_device_no_range, &abs_mt_range);
-	litest_add_no_device("device:invalid devices", abs_device_missing_res);
-	litest_add_no_device("device:invalid devices", abs_mt_device_missing_res);
-	litest_add_no_device("device:invalid devices", ignore_joystick);
+	litest_add_no_device(abs_device_no_absx);
+	litest_add_no_device(abs_device_no_absy);
+	litest_add_no_device(abs_mt_device_no_absx);
+	litest_add_no_device(abs_mt_device_no_absy);
+	litest_add_ranged_no_device(abs_device_no_range, &abs_range);
+	litest_add_ranged_no_device(abs_mt_device_no_range, &abs_mt_range);
+	litest_add_no_device(abs_device_missing_res);
+	litest_add_no_device(abs_mt_device_missing_res);
+	litest_add_no_device(ignore_joystick);
 
-	litest_add("device:wheel", device_wheel_only, LITEST_WHEEL, LITEST_RELATIVE|LITEST_ABSOLUTE|LITEST_TABLET);
-	litest_add_no_device("device:accelerometer", device_accelerometer);
+	litest_add(device_wheel_only, LITEST_WHEEL, LITEST_RELATIVE|LITEST_ABSOLUTE|LITEST_TABLET);
+	litest_add_no_device(device_accelerometer);
 
-	litest_add("device:udev tags", device_udev_tag_wacom_tablet, LITEST_TABLET, LITEST_TOTEM);
+	litest_add(device_udev_tag_wacom_tablet, LITEST_TABLET, LITEST_TOTEM);
 
-	litest_add_no_device("device:invalid rel events", device_nonpointer_rel);
-	litest_add_no_device("device:invalid rel events", device_touchpad_rel);
-	litest_add_no_device("device:invalid rel events", device_touch_rel);
-	litest_add_no_device("device:invalid rel events", device_abs_rel);
+	litest_add_no_device(device_nonpointer_rel);
+	litest_add_no_device(device_touchpad_rel);
+	litest_add_no_device(device_touch_rel);
+	litest_add_no_device(device_abs_rel);
 
-	litest_add_for_device("device:quirks", device_quirks_no_abs_mt_y, LITEST_ANKER_MOUSE_KBD);
-	litest_add_for_device("device:quirks", device_quirks_cyborg_rat_mode_button, LITEST_CYBORG_RAT);
-	litest_add_for_device("device:quirks", device_quirks_apple_magicmouse, LITEST_MAGICMOUSE);
-	litest_add_for_device("device:quirks", device_quirks_logitech_marble_mouse, LITEST_LOGITECH_TRACKBALL);
+	litest_add_for_device(device_quirks_no_abs_mt_y, LITEST_ANKER_MOUSE_KBD);
+	litest_add_for_device(device_quirks_cyborg_rat_mode_button, LITEST_CYBORG_RAT);
+	litest_add_for_device(device_quirks_apple_magicmouse, LITEST_MAGICMOUSE);
+	litest_add_for_device(device_quirks_logitech_marble_mouse, LITEST_LOGITECH_TRACKBALL);
+	litest_add_no_device(device_quirks);
 
-	litest_add("device:capability", device_capability_at_least_one, LITEST_ANY, LITEST_ANY);
-	litest_add("device:capability", device_capability_check_invalid, LITEST_ANY, LITEST_ANY);
-	litest_add_no_device("device:capability", device_capability_nocaps_ignored);
+	litest_add(device_capability_at_least_one, LITEST_ANY, LITEST_ANY);
+	litest_add(device_capability_check_invalid, LITEST_ANY, LITEST_ANY);
+	litest_add_no_device(device_capability_nocaps_ignored);
 
-	litest_add("device:size", device_has_size, LITEST_TOUCHPAD, LITEST_ANY);
-	litest_add("device:size", device_has_size, LITEST_TABLET, LITEST_ANY);
-	litest_add("device:size", device_has_no_size, LITEST_ANY,
+	litest_add(device_has_size, LITEST_TOUCHPAD, LITEST_ANY);
+	litest_add(device_has_size, LITEST_TABLET, LITEST_ANY);
+	litest_add(device_has_no_size, LITEST_ANY,
 		   LITEST_TOUCHPAD|LITEST_TABLET|LITEST_TOUCH|LITEST_ABSOLUTE|LITEST_SINGLE_TOUCH|LITEST_TOTEM);
 
-	litest_add_for_device("device:output", device_get_output, LITEST_CALIBRATED_TOUCHSCREEN);
-	litest_add("device:output", device_no_output, LITEST_RELATIVE, LITEST_ANY);
-	litest_add("device:output", device_no_output, LITEST_KEYS, LITEST_ANY);
+	litest_add_for_device(device_get_output, LITEST_CALIBRATED_TOUCHSCREEN);
+	litest_add(device_no_output, LITEST_RELATIVE, LITEST_ANY);
+	litest_add(device_no_output, LITEST_KEYS, LITEST_ANY);
 
-	litest_add("device:seat", device_seat_phys_name, LITEST_ANY, LITEST_ANY);
+	litest_add(device_seat_phys_name, LITEST_ANY, LITEST_ANY);
 
-	litest_add("device:button", device_button_down_remove, LITEST_BUTTON, LITEST_ANY);
+	litest_add(device_button_down_remove, LITEST_BUTTON, LITEST_ANY);
 }
