@@ -485,7 +485,7 @@ find_device(const char *udev_tag)
 			continue;
 
 		sysname = udev_device_get_sysname(device);
-		if (strncmp("event", sysname, 5) != 0) {
+		if (!strneq("event", sysname, 5)) {
 			udev_device_unref(device);
 			continue;
 		}
